/*
// shatbox (C) 2002 Matthew Conte (matt@conte.com)
**
**
** config.h
**
** Configuration API 
**
** This file was written by Ben Sittler (bsittler@iname.com)
** and is in the public domain.
**
** $Id: $
*/

#ifndef _CONFIG_H_
#define _CONFIG_H_

#include "version"

#ifndef CONFIG_FILE
#define CONFIG_FILE APPNAME".cfg"
#endif

struct myvar_t
{
   struct myvar_t *less, *greater;
   char *group, *key, *value;
};

class config
{
private:
   myvar_t  *mpVars;
   bool     mSaveNeeded;

public:
   config();
   ~config();
   
   /* open loads from the disk the saved configuration.
   **
   ** open must be the first config function called.
   **
   ** open returns true on success, false otherwise.
   */
   int   open();
      
   /* close saves the current configuration to disk.
   **
   ** close must be the last config function called.
   */
   int   close();

   /* read_int loads an integer from the configuration into "value"
   **
   ** If the specified "key" does not exist, the "def"ault is returned
   */
   int   read_int(const char *group, const char *key, int def);

   /* read_string copies a string from the configuration into "value"
   **
   ** If the specified "key" does not exist, the "def"ault is returned
   */
   const char *read_string(const char *group, const char *key, const char *def);
      
   void  write_int(const char *group, const char *key, int value);
   void  write_string(const char *group, const char *key, const char *value);

};

#endif /* !_CONFIG_H_ */
