//
// shatbox (C) 2002 Matthew Conte (matt@conte.com)
//
//
// input.cpp
//
// nes input routines
//
// $Id: $
//

#include <string.h>
#include "assert.h"
#include "input.h"
#include "regs.h"
#include "ppu.h"
#include "cpu.h"
#include "event.h"
#include "nes6502.h"

static const nes6502_memread input_memread[] =
{
   { 0x4016, 0x4017, input::dispatchRead, NULL },
   { (uint32) -1, (uint32) -1, NULL, NULL }
};

static const nes6502_memwrite input_memwrite[] = 
{
   { 0x4016, 0x4017, input::dispatchWrite, NULL },
   { (uint32) -1, (uint32) -1, NULL, NULL }
};


input::input(machine *pMachine)
: unit(pMachine)
{
   mpMemRead = input_memread;
   mpMemWrite = input_memwrite;

   mStrobe = 0;
   memset(mPad, 0, sizeof(mPad));
}


void input::portStrobe()
{
   for (int i = 0; i < 2; i++)
      mPad[i].readCount = 0;
}


void input::portWrite(uint8 value)
{
   value &= 1;

   if (0 == value && mStrobe)
      portStrobe();

   mStrobe = value;
}


uint8 input::portRead(int port)
{
   /* avoid mutually exclusive output */
   uint8 value = mPad[port].state;

   if (value & (1 << PAD_UP))
      value &= ~(1 << PAD_DOWN);
   if (value & (1 << PAD_RIGHT))
      value &= ~(1 << PAD_LEFT);

   return 0x40 | ((value >> mPad[port].readCount++) & 1);
}


void input::trigger(eInputDevice device, uint8 value, eInputState state)
{
   int port = (int) device;
   
   if (INPUT_RELEASE == state)
      mPad[port].state &= ~value;
   else
      mPad[port].state |= value;
}


uint8 input::dispatchRead(void *userdata, uint32 offset)
{
   return ((input *) userdata)->read(offset);
}


void input::dispatchWrite(void *userdata, uint32 offset, uint8 value)
{
   ((input *) userdata)->write(offset, value);
}


uint8 input::read(uint32 offset)
{
   ASSERT(offset < 2);

   return portRead(offset);
}


void input::write(uint32 offset, uint8 value)
{
   ASSERT(offset < 2);

   switch (offset)
   {
   case 0:
      // TODO: vs unisystem prg switching

      // check for joypad strobe
      portWrite(value);
      break;

   case 1:

      // TODO: nes frame irq control
      break;

   default:
      break;
   }
}
