/*
// shatbox (C) 2002 Matthew Conte (matt@conte.com)
**
**
** map004.cpp
**
** mapper 4 implementation
**
** $Id: $
*/

#include "ppu.h"
#include "cpu.h"
#include "nes.h"
#include "mmc/map004.h"


void map004::write(uint32 offset, uint8 value)
{
   switch (offset & 0x6001)
   {
   case 0x0000:
      mCommand = value;
      mVROMBase = (value & 0x80) ? 0x1000 : 0x0000;

      if (mReg != (value & 0x40))
      {
         uint16 base = (value & 0x40) ? 0x8000 : 0xc000;
         
         // TODO: hmm....
         //prgbank(8, base, CART_LASTBANK);
         prgbank(8, base, mInfo.mNumPRG * 2 - 2);
      }

      mReg = value & 0x40;
      break;

   case 0x0001:
      switch (mCommand & 7)
      {
      case 0:
         value &= 0xfe;
         chrbank(1, mVROMBase ^ 0x0000, value);
         chrbank(1, mVROMBase ^ 0x0400, value + 1);
         break;

      case 1:
         value &= 0xfe;
         chrbank(1, mVROMBase ^ 0x0800, value);
         chrbank(1, mVROMBase ^ 0x0c00, value + 1);
         break;

      case 2:
         chrbank(1, mVROMBase ^ 0x1000, value);
         break;

      case 3:
         chrbank(1, mVROMBase ^ 0x1400, value);
         break;

      case 4:
         chrbank(1, mVROMBase ^ 0x1800, value);
         break;

      case 5:
         chrbank(1, mVROMBase ^ 0x1c00, value);
         break;

      case 6:
         prgbank(8, (mCommand & 0x40) ? 0xc000 : 0x8000, value);
         break;

      case 7:
         prgbank(8, 0xa000, value);
         break;
      }
      break;

   case 0x2000:
      /* four screen mirroring? */
      if (MIRROR_FOUR != mInfo.mMirror)
      {
         if (value & 1)
            ((NES *) mpMachine)->getPPU()->mirror(0, 0, 1, 1);
         else
            ((NES *) mpMachine)->getPPU()->mirror(0, 1, 0, 1);
      }
      break;

   case 0x2001:
      /* TODO: unused? */
      break;

   case 0x4000:
      mIRQCounter = value;
      break;

   case 0x4001:
      mIRQLatch = value;
      break;

   case 0x6000:
      mIRQEnabled = false;
      break;

   case 0x6001:
      mIRQEnabled = true;
      break;

   default:
      break;
   }
}


void map004::dispatchWrite(void *userdata, uint32 offset, uint8 value)
{
   ((map004 *) userdata)->write(offset, value);
}


static nes6502_memwrite map004_memwrite[] =
{
   { 0x8000, 0xffff, map004::dispatchWrite, NULL },
   { (uint32) -1, (uint32) -1, NULL, NULL }
};


map004::map004(machine *pMachine, mapperInfo *pInfo)
: mmc(pMachine, pInfo)
{
   mpName = "Nintendo MMC3";
   mpMemWrite = map004_memwrite;

   resetRegs();
}


void map004::resetRegs()
{
   mReg = 0;
   mCommand = 0;
   mVROMBase = 0x0000;

   mIRQCounter = 0;
   mIRQLatch = 0;
   mIRQEnabled = false;
}


void map004::reset(bool hard_reset)
{
   mmc::reset(hard_reset);
   resetRegs();
}


void map004::hblank(bool in_vblank)
{
   if (in_vblank)
      return;

   if (((NES *) mpMachine)->getPPU()->enabled() && mIRQEnabled)
   {
      if (0 == mIRQCounter)
      {
         mIRQCounter = mIRQLatch;
         ((NES *) mpMachine)->getCPU()->irq();
      }
      else
      {
         mIRQCounter--;
      }
   }
}

void map004::getState(SnssMapperBlock *pState)
{
   pState->extraData.mapper4.irqCounter = mIRQCounter;
   pState->extraData.mapper4.irqLatchCounter = mIRQLatch;
   pState->extraData.mapper4.irqCounterEnabled = mIRQEnabled;
   pState->extraData.mapper4.last8000Write = mCommand;
}


void map004::setState(SnssMapperBlock *pState)
{
   mIRQCounter = pState->extraData.mapper4.irqCounter;
   mIRQLatch = pState->extraData.mapper4.irqLatchCounter;
   mIRQEnabled = pState->extraData.mapper4.irqCounterEnabled;
   mCommand = pState->extraData.mapper4.last8000Write;
}
