//
// shatbox (C) 2002 Matthew Conte (matt@conte.com)
//
//
// sys.h
//
// System-dependent routines
//
// $Id: $
//

#ifndef _SYS_H_
#define _SYS_H_

#include "typedefs.h"

class config;
class eventHandler;
class System
{
public:
   System();
   virtual ~System();

   int   getSampleRate() const;
   virtual void  silenceAudio();

   virtual void  lockVideo();
   virtual void  unlockVideo();
   virtual void  setColor(int index, int r, int g, int b, int value = -1) = 0;

   virtual void  submitAudio(unsigned char *pStream, int numSamples);
   virtual void  submitVideo(unsigned char *pBuffer, int scanline);

   virtual int   pollEvents(eventHandler *pEventHandler) = 0;

   virtual void  mark();
   virtual int   elapsed();
   virtual void  usleep(int us);
   virtual int   timerResolution() const;

protected:

   void  audioCallback(unsigned char *pStream, int numSamples);
   
   virtual void disableInterrupts() {}
   virtual void enableInterrupts() {}

   config   *mpConfig;
   
   bool     mSoundEnabled;
   unsigned char *mpAudioStream;
   int      mSampleRate;
   int      mAudioFragSize;
   volatile int   mAudioPosition;
   volatile int   mAudioAvailable;

   unsigned char *mpPixels;
   int      mWidth;
   int      mHeight;
   int      mBPP;
   int      mScale;
   int      mPitch;
   int      mPixelWidth;
   int      mLoss[3];   // rgb pixel right shift offsets
   int      mPos[3];    // rgb pixel left shit offsets
   bool     mIndexed;

   int      mVideoLock;

   long long mTimer;

   uint32   mPalette4[32];
   uint16   mPalette2[32];
   uint8    mPalette1[32];
};

inline int System::getSampleRate() const
{
   return mSampleRate;
}


extern System *System_factory();

#endif // !_SYS_H_
